<!-- Advanced Reports Styling -->
<link rel="stylesheet" type="text/css" href="{{ STATIC_URL }}advanced_reports/css/advreport.css" />
<link rel="stylesheet" type="text/css" href="{{ STATIC_URL }}advanced_reports/css/show-notice.css"/>
<link rel="stylesheet" type="text/css" href="{{ STATIC_URL }}advanced_reports/css/ui.datepicker.css"/>
<link rel="stylesheet" type="text/css" href="{{ STATIC_URL }}advanced_reports/css/jquery.mbox.css"/>
<!-- / Advanced Reports Styling -->