{% extends "advanced_reports/backoffice/base.html" %}

{% block header-title-wrapper %}
    <header class="main-header">
        <div class="container">
            <h1>{% block page-title %}Page title{% endblock %}</h1>
        </div>
    </header>
{% endblock %}
