{% extends "advanced_reports/backoffice/tests/page-base.html" %}

{% block page-title %}Users{% endblock %}

{% block content %}

<div class="container">
    <h1>Page</h1>

    <div view="simple" extra="true"></div>

    <div class="panel panel-default">
        <div class="panel-heading">
            <h2>Users</h2>
        </div>
        <div view="advanced_report" params="{slug: 'user'}"></div>
    </div>
</div>

{% endblock %}
