<div class="panel panel-default">
    <div class="panel-heading">
        <h2>Edit todo item</h2>
    </div>
       <div class="panel-body">
        <form class="form-horizontal"
              ng-submit="call('edit', {todo_item: model.id, name: model.name}).then(goto(model.parents[0].route))">
            <div class="form-group">
                <label for="inputName" class="col-sm-2 control-label">Name</label>

                <div class="col-sm-10">
                    <input type="text" class="form-control" id="inputName" placeholder="Name" ng-model="model.name">
                </div>
            </div>
            <div class="form-group">
                <div class="col-sm-offset-2 col-sm-10">
                    <button type="submit" class="btn btn-primary">Save</button>
                </div>
            </div>
        </form>
    </div>
</div>
