<form post-to-view role="form" class="well form-horizontal">
    {% include "advanced_reports/backoffice/contrib/advanced-reports/bootstrap-form.html" %}
    <div class="form-group">
        <div class="col-md-9 col-md-offset-3">
            <input class="btn btn-primary" type="submit" value="Save">
            <input class="btn btn-default" type="button" value="Refresh page" ng-click="fetchModel(true)">
        </div>
    </div>
</form>
