{% load i18n %}
{% verbatim %}
<script type="text/ng-template" id="/modal.html">
    <div class="modal fade" tabindex="-1" role="dialog" bo-element="boModal">
        <div class="modal-dialog-extra-padding modal-dialog" style="{{ modalStyle }}">
            <div class="modal-content">
                <div class="modal-header">
                    <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>
                    <h4 class="modal-title" ng-bind="modalTitle"></h4>
                </div>
                <div ng-transclude></div>
            </div><!-- /.modal-content -->
        </div><!-- /.modal-dialog -->
    </div><!-- /.modal -->
</script>
{% endverbatim %}
