{% extends "helpdesk/base.html" %}

{% block page_title %}{{ block.super }}{% endblock %}
{% block title %}{{ block.super }}{% endblock %}

{% block extra_head %}
    {{ block.super }}

    {% include "advanced_reports/inc_css.html" %}
{% endblock %}

{% block extra_footer %}
    {{ block.super }}
    
    {% include "advanced_reports/inc_js.html" %}
{% endblock %}
