{% extends "advanced_reports/backoffice/model-base.html" %}

{% block title %}Oemfoe Todos Administration{% endblock %}
{% block head-title %}Oemfoe Todos Administration{% endblock %}

{% block header-title-wrapper %}
    <header class="main-header" ng-hide="useView()">
        <div class="container">
            <h1>{% block page-title %}{{ backoffice.title }}{% endblock %}</h1>
        </div>
    </header>
{% endblock %}

{% block navbar %}
<li class="dropdown">
    <a href="#" class="dropdown-toggle" data-toggle="dropdown">Navigation <b class="caret"></b></a>
    <ul class="dropdown-menu">
        <li><a href="{% url 'todos:users' %}">Users</a></li>
        <li><a href="{% url 'todos:examples' %}">Examples</a></li>
    </ul>
</li>
{% endblock %}
