{% extends "advanced_reports/base.html" %}

{% load i18n pagination %}

{% block page_title %}{{ advreport.title }} - {{ block.super }}{% endblock %}
{% block title %}{{ advreport.title }}{% endblock %}

{% block content %}
    {% include "advanced_reports/inc_searchitems_content.html" %}
{% endblock %}
