.information-row td { cursor: hand; cursor: pointer; }
.inline p, .inline ul, .inline li { display: inline; }
.loader { display: none; width: 16px; height: 16px; }
.action-form p, .header-form p { margin: 0; }
.limit-width { width: 400px; }
.item-row .action-link {margin-left: 60%;}
.collapse-form p,
.collapse-form table,
.collapse-form tr,
.collapse-form td,
.collapse-form th { display: inline; }
